"""
Type hint classes for the representation of the json state.
"""
from datetime import datetime

from pydantic import BaseModel
from typing_extensions import Literal, TypedDict


class OrderState(TypedDict):
    id: str
    category: Literal["Order"]
    meal: str
    start_time: datetime  # isoformat str
    max_duration: float


class EffectState(TypedDict):
    id: str
    type: str
    progress_percentage: float | int
    inverse_progress: bool


class ItemState(TypedDict):
    id: str
    category: Literal["Item"] | Literal["ItemCookingEquipment"]
    type: str
    progress_percentage: float | int
    inverse_progress: bool
    active_effects: list[EffectState]
    # add ItemType Meal ?


class CookingEquipmentState(TypedDict):
    content_list: list[ItemState]
    content_ready: None | ItemState


class CounterState(TypedDict):
    id: str
    category: Literal["Counter"]
    type: str
    pos: list[float]
    orientation: list[float]
    occupied_by: None | list[
        ItemState | CookingEquipmentState
    ] | ItemState | CookingEquipmentState
    active_effects: list[EffectState]
    # list[ItemState] -> type in ["Sink", "PlateDispenser"]


class PlayerState(TypedDict):
    id: str
    pos: list[float]
    facing_direction: list[float]
    holding: ItemState | CookingEquipmentState | None
    current_nearest_counter_pos: list[float] | None
    current_nearest_counter_id: str | None


class KitchenInfo(BaseModel):
    """Basic information of the kitchen."""

    width: float
    height: float


class ViewRestriction(BaseModel):
    direction: list[float]
    position: list[float]
    angle: int  # degrees
    counter_mask: None | list[bool]
    range: float | None


class StateRepresentation(BaseModel):
    """The format of the returned state representation."""

    players: list[PlayerState]
    counters: list[CounterState]
    kitchen: KitchenInfo
    score: float | int
    orders: list[OrderState]
    ended: bool
    env_time: datetime  # isoformat str
    remaining_time: float
    view_restriction: None | ViewRestriction


def create_json_schema():
    return StateRepresentation.model_json_schema()


if __name__ == "__main__":
    print(create_json_schema())
